@@ -27,16 +27,16 @@ from contextlib import ExitStack, contextmanager
 
 import testflows.settings as settings
 
-from .exceptions import DummyTestException, ResultException, RepeatTestException
+from .exceptions import DummyTestException, ResultException, RepeatTestException, DescriptionError
 from .flags import Flags, SKIP, TE, FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NOT_COUNTED
 from .flags import CFLAGS, PAUSE_BEFORE, PAUSE_AFTER
 from .testtype import TestType, TestSubType
-from .objects import get, Null, OK, Fail, Skip, Error, PassResults, Argument, Attribute, ExamplesTable
+from .objects import get, Null, OK, Fail, Skip, Error, PassResults, Argument, Attribute, Requirement
+from .objects import RepeatTest, ExamplesTable
 from .constants import name_sep, id_sep
 from .io import TestIO, LogWriter
 from .name import join, depth, match, absname
 from .funcs import current, top, previous, main, exception, pause
-from .funcs import xfails, xflags, repeat
 from .init import init
 from .cli.arg.parser import ArgumentParser as ArgumentParserClass
 from .cli.arg.common import epilog as common_epilog
@@ -44,7 +44,7 @@ from .cli.arg.exit import ExitWithError, ExitException
 from .cli.arg.type import key_value as key_value_type, repeat as repeat_type, tags_filter as tags_filter_type
 from .cli.text import danger, warning
 from .exceptions import exception as get_exception
-from .filters import the, thetags
+from .filters import The, TheTags
 from .utils.sort import human as human_sort
 
 try:
@@ -199,11 +199,13 @@ class TestBase(object):
         self.subtype = get(subtype, self.subtype)
         self.context = get(context, current().context if current() and self.type < TestType.Iteration else (Context(current().context if current() else None)))
         self.tags = tags
-        self.requirements = {r.name: r for r in get(requirements, list(self.requirements))}
-        self.attributes =  {a.name: a for a in get(attributes, list(self.attributes))}
+        self.requirements = {r.name: r for r in [Requirement(*r) for r in get(requirements, list(self.requirements))]}
+        self.attributes =  {a.name: a for a in [Attribute(*a) for a in get(attributes, list(self.attributes))]}
         self.args = {k: Argument(k, v) for k,v in get(args, {}).items()}
         self.description = get(description, self.description)
         self.examples = get(examples, get(self.examples, ExamplesTable()))
+        if not isinstance(self.examples, ExamplesTable):
+            self.examples = ExamplesTable(*self.examples)
         self.result = Null(test=self.name)
         if flags is not None:
             self.flags = Flags(flags)
@@ -241,6 +243,15 @@ class TestBase(object):
         name = name.replace(name_sep, "\u2215")
         return join(get(parent, name_sep), name)
 
+    @classmethod
+    def make_description(cls, description, args):
+        if args is None:
+            args = dict()
+        try:
+            return str(description).format(**{"$cls": cls}, **args) if description is not None else None
+        except Exception as exc:
+            raise DescriptionError(f"can't format '{description}' using {args} {str(exc)}") from None
+
     @classmethod
     def make_tags(cls, tags):
         return set(get(tags, cls.tags))
@@ -250,6 +261,7 @@ class TestBase(object):
         if top() is self:
             self.io.output.protocol()
             self.io.output.version()
+
         self.io.output.test_message()
 
         if self.flags & PAUSE_BEFORE:
@@ -297,7 +309,7 @@ class TestBase(object):
                         if not exc_value and cleanup_exc_value:
                             process_exception(cleanup_exc_type, cleanup_exc_value, cleanup_exc_traceback)
             finally:
-                current(self.caller_test)
+                current(self.caller_test, set_value=True)
                 previous(self)
                 self._apply_xfails()
                 self.io.output.result(self.result)
@@ -376,6 +388,13 @@ class TestDefinition(object):
     def __new__(cls, name=None, **kwargs):
         run = kwargs.pop("run", None)
         test = kwargs.pop("test", None)
+        no_arguments = None
+
+        if run and test:
+            raise TypeError("can't define both test and run")
+
+        if kwargs.get("args", None):
+            no_arguments = False
 
         if name is not None:
             kwargs["name"] = name
@@ -396,9 +415,8 @@ class TestDefinition(object):
                 kwargs["test"] = run
             else:
                 raise TypeError(f"'{run}' is not a valid test type")
-            return cls.__create__(**kwargs)()
 
-        if test:
+        elif test:
             if isinstance(test, TestDecorator):
                 kwargs = inherit_kwargs(**test.func.kwargs)
                 kwargs["test"] = test
@@ -409,7 +427,12 @@ class TestDefinition(object):
             else:
                 raise TypeError(f"'{test}' is not a valid test type")
 
-        return cls.__create__(**kwargs)
+        self = cls.__create__(**kwargs)
+        self.no_arguments = no_arguments
+
+        if run:
+            return self()
+        return self
 
     @classmethod
     def __create__(cls,  **kwargs):
@@ -465,7 +488,7 @@ class TestDefinition(object):
                             help="pause before executing selected tests", type=str, required=False)
         parser.add_argument("--pause-after", dest="_pause_after", metavar="pattern", nargs="+",
                             help="pause after executing selected tests", type=str, required=False)
-        parser.add_argument("--random-order", dest="_random_order", action="store_true", 
+        parser.add_argument("--random", dest="_random_order", action="store_true",
                             help="randomize order of auto loaded tests", required=False)
         parser.add_argument("--debug", dest="_debug", action="store_true",
                             help="enable debugging mode", default=False)
@@ -545,7 +568,7 @@ class TestDefinition(object):
                 args.pop("_random_order")
 
             if args.get("_pause_before"):
-                xflags = kwargs.get("xflags", globals()["xflags"]())
+                xflags = kwargs.get("xflags", {})
                 for pattern in args.get("_pause_before"):
                     xflags[pattern] = xflags.get(pattern, [0, 0])
                     xflags[pattern][0] |= PAUSE_BEFORE
@@ -553,7 +576,7 @@ class TestDefinition(object):
                 args.pop("_pause_before")
 
             if args.get("_pause_after"):
-                xflags = kwargs.get("xflags", globals()["xflags"]())
+                xflags = kwargs.get("xflags", {})
                 for pattern in args.get("_pause_after"):
                     xflags[pattern] = xflags.get(pattern, [0, 0])
                     xflags[pattern][0] |= PAUSE_AFTER
@@ -563,34 +586,34 @@ class TestDefinition(object):
             if args.get("_only"):
                 only = []
                 for pattern in args.pop("_only"):
-                    only.append(the(pattern))
+                    only.append(The(pattern))
                 kwargs["only"] = only
 
             if args.get("_skip"):
                 skip = []
                 for pattern in args.pop("_skip"):
-                    skip.append(the(pattern))
+                    skip.append(The(pattern))
                 kwargs["skip"] = skip
 
             if args.get("_start"):
                 pattern = args.pop("_start")[0]
-                kwargs["start"] = the(pattern)
+                kwargs["start"] = The(pattern)
 
             if args.get("_end"):
                 pattern = args.pop("_end")[0]
-                kwargs["end"] = the(pattern)
+                kwargs["end"] = The(pattern)
 
             if args.get("_only_tags"):
                 _only_tags = {}
                 for item in args.pop("_only_tags"):
                     _only_tags.update(item)
-                kwargs["only_tags"] = thetags(**_only_tags)
+                kwargs["only_tags"] = TheTags(**_only_tags)
 
             if args.get("_skip_tags"):
                 _skip_tags = {}
                 for item in args.pop("_skip_tags"):
                     _skip_tags.update(item)
-                kwargs["skip_tags"] = thetags(**_skip_tags)
+                kwargs["skip_tags"] = TheTags(**_skip_tags)
 
             if args.get("_tags"):
                 kwargs["tags"] = {value for value in args.pop("_tags")}
@@ -616,9 +639,11 @@ class TestDefinition(object):
 
     def __call__(self, **args):
         test = self.kwargs.get("test", None)
-        self.kwargs["args"] = self.kwargs.get("args", {})
+        self.kwargs["args"] = dict(self.kwargs.get("args") or {})
         self.kwargs["args"].update(args)
 
+        self.no_arguments = get(self.no_arguments, not args)
+
         if test and isinstance(test, TestDecorator):
             self.repeatable_func = test
             with self as _test:
@@ -634,6 +659,7 @@ class TestDefinition(object):
             pass
         try:
             kwargs = self.kwargs
+            kwargs["args"] = dict(kwargs.get("args") or {})
 
             argparser = kwargs.pop("argparser", None)
             parent = kwargs.pop("parent", None) or current()
@@ -645,24 +671,26 @@ class TestDefinition(object):
             self._set_current_top_previous()
 
             test = kwargs.pop("test", None)
+            kwargs_test = test
             if test and isinstance(test, TestDecorator):
                 test = test.func.kwargs.get("test", None)
             test = test if test is not None else TestBase
             if not issubclass(test, TestBase):
                 raise TypeError(f"{test} must be subclass of TestBase")
-            name = test.make_name(self.name, parent.name if parent else None, kwargs.get("args", {}))
+
+            name = test.make_name(self.name, parent.name if parent else None, kwargs["args"])
 
             if parent:
                 kwargs["parent"] = parent.name
                 kwargs["id"] = parent.id + [parent.child_count]
                 kwargs["cflags"] = parent.cflags
                 # propagate xfails, xflags that prefix match the name of the test
-                kwargs["xfails"] = xfails({
+                kwargs["xfails"] = {
                     k: v for k, v in parent.xfails.items() if match(name, k, prefix=True)
-                }) if parent.xfails else None or kwargs.get("xfails")
-                kwargs["xflags"] = xflags({
+                } if parent.xfails else None or kwargs.get("xfails")
+                kwargs["xflags"] = {
                     k: v for k, v in parent.xflags.items() if match(name, k, prefix=True)
-                }) if parent.xfails else None or kwargs.get("xflags")
+                } if parent.xflags else None or kwargs.get("xflags")
                 # propagate only, skip, start, and end
                 kwargs["only"] = parent.only or kwargs.get("only")
                 kwargs["skip"] = parent.skip or kwargs.get("skip")
@@ -671,7 +699,7 @@ class TestDefinition(object):
                 kwargs["only_tags"] = parent.only_tags or kwargs.get("only_tags")
                 kwargs["skip_tags"] = parent.skip_tags or kwargs.get("skip_tags")
                 # propagate repeat
-                if parent.repeat and parent.type > TestType.Test and self.type >= TestType.Test:
+                if parent.repeat and parent.type > TestType.Outline and self.type >= TestType.Outline:
                     kwargs["repeat"] = parent.repeat
                 # handle parent test type propagation
                 if keep_type is None:
@@ -680,35 +708,56 @@ class TestDefinition(object):
                     parent.child_count += 1
 
             self.tags = test.make_tags(kwargs.pop("tags", None))
+            self.description = test.make_description(kwargs.pop("description", None), kwargs["args"])
             self.parent = parent
 
             # anchor all patterns
-            kwargs["xfails"] = xfails({
-                absname(k, name if name else name_sep): v for k, v in (kwargs.get("xfails") or {}).items()
-            }) or None
-            kwargs["xflags"] = xflags({
-                absname(k, name if name else name_sep): v for k, v in (kwargs.get("xflags") or {}).items()
-            }) or None
-            kwargs["only"] = [f.at(name if name else name_sep) for f in kwargs.get("only") or []] or None
-            kwargs["skip"] = [f.at(name if name else name_sep) for f in kwargs.get("skip") or []] or None
-            kwargs["start"] = kwargs.get("start").at(name if name else name_sep) if kwargs.get("start") else None
-            kwargs["end"] = kwargs.get("end").at(name if name else name_sep) if kwargs.get("end") else None
-            kwargs["repeat"] = [globals()["repeat"](r.pattern.at(name if name else name_sep),r.number,r.until) for r in kwargs.get("repeat", [])] or None
+            kwargs["xfails"] = {
+                absname(k, name if name else name_sep): v for k, v in dict(kwargs.get("xfails") or {}).items()
+            } or None
+            kwargs["xflags"] = {
+                absname(k, name if name else name_sep): v for k, v in dict(kwargs.get("xflags") or {}).items()
+            } or None
+            kwargs["only"] = [The(str(f)).at(name if name else name_sep) for f in kwargs.get("only") or []] or None
+            kwargs["skip"] = [The(str(f)).at(name if name else name_sep) for f in kwargs.get("skip") or []] or None
+            kwargs["start"] = The(str(kwargs.get("start"))).at(name if name else name_sep) if kwargs.get("start") else None
+            kwargs["end"] = The(str(kwargs.get("end"))).at(name if name else name_sep) if kwargs.get("end") else None
+            kwargs["only_tags"] = TheTags(**dict(kwargs["only_tags"])) if kwargs.get("only_tags") and not isinstance(kwargs["only_tags"], TheTags) else None
+            kwargs["skip_tags"] = TheTags(**dict(kwargs["skip_tags"])) if kwargs.get("skip_tags") and not isinstance(kwargs["skip_tags"], TheTags) else None
+            kwargs["repeat"] = [RepeatTest(*r) for r in kwargs.get("repeat", [])] or None
+            if kwargs["repeat"]:
+                [r.pattern.at(name if name else name_sep) for r in kwargs["repeat"]]
+            # should not skip Background, Given and Finally steps
+            if kwargs.get("subtype") in (TestSubType.Background, TestSubType.Given, TestSubType.Finally):
+                kwargs["only"] = kwargs["only"] or []
+                kwargs["only"].append(The(join(name, "*")))
 
             self._apply_xflags(name, kwargs)
             self._apply_start(name, parent, kwargs)
             self._apply_only_tags(self.type, self.tags, kwargs)
             self._apply_skip_tags(self.type, self.tags, kwargs)
-            self._apply_only(name, kwargs)
             self._apply_skip(name, kwargs)
             self._apply_end(name, parent, kwargs)
+            self._apply_only(name, kwargs)
+
+            if not top():
+                # can't skip, pause before or after top level test
+                kwargs["flags"] = Flags(kwargs.get("flags")) & ~SKIP
+                kwargs["flags"] &= ~PAUSE_BEFORE
+                kwargs["flags"] &= ~PAUSE_AFTER
 
             self.repeat = kwargs.pop("repeat", None)
 
-            self.test = test(name, tags=self.tags, repeat=self.repeat, **kwargs)
+            self.test = test(name, tags=self.tags, description=self.description, repeat=self.repeat, **kwargs)
             if getattr(self, "parent_type", None):
                 self.test.parent_type = self.parent_type
 
+            # indicate that parent is running an outline
+            # and if there are any user arguments for an outline
+            if isinstance(kwargs_test, TestOutline):
+                self.test._run_outline_with_no_arguments = self.no_arguments
+                self.test._run_outline = True
+
             if self.repeat is not None:
                 repeat = self._apply_repeat(name, self.repeat)
                 if repeat is not None:
@@ -746,7 +795,7 @@ class TestDefinition(object):
             if parent:
                 with parent.lock:
                     parent.end = None
-                    parent.skip = [the("/*")]
+                    parent.skip = [The("/*")]
 
     def _apply_start(self, name, parent, kwargs):
         start = kwargs.get("start")
@@ -757,6 +806,7 @@ class TestDefinition(object):
             kwargs["flags"] = Flags(kwargs.get("flags")) | SKIP
         else:
             kwargs["flags"] = Flags(kwargs.get("flags")) & ~SKIP
+            kwargs["start"] = None
             if parent:
                 with parent.lock:
                     parent.start = None
@@ -766,7 +816,7 @@ class TestDefinition(object):
             return
 
         for item in repeat:
-            if item.pattern.match(name,prefix=False):
+            if item.pattern.match(name, prefix=False):
                 return item
 
     def _apply_only_tags(self, type, tags, kwargs):
@@ -792,10 +842,6 @@ class TestDefinition(object):
         if not only:
             return
 
-        # only should not skip Background, Given and Finally steps
-        if kwargs.get("subtype") in (TestSubType.Background, TestSubType.Given, TestSubType.Finally):
-            only.append(the(join(name, "*")))
-
         found = False
         for item in only:
             if item.match(name):
@@ -804,6 +850,8 @@ class TestDefinition(object):
 
         if not found:
             kwargs["flags"] = Flags(kwargs.get("flags")) | SKIP
+        else:
+            kwargs["flags"] = Flags(kwargs.get("flags")) & ~SKIP
 
     def _apply_skip(self, name, kwargs):
         skip = kwargs.get("skip")
@@ -878,6 +926,9 @@ class TestDefinition(object):
                     __kwargs["flags"] = Flags(__kwargs.get("flags")) | TE
                 for i in range(repeat.number):
                     with Iteration(name=f"{i}", tags=self.tags, **__kwargs, parent_type=self.test.type) as iteration:
+                        if isinstance(self.repeatable_func, TestOutline):
+                            iteration._run_outline_with_no_arguments = self.no_arguments
+                            iteration._run_outline = True
                         self.repeatable_func(**__args)
                     if repeat.until == "pass" and isinstance(iteration.result, PassResults):
                         break
@@ -901,10 +952,10 @@ class TestDefinition(object):
         if exception_value and not test__exit__:
             raise exception_value.with_traceback(exception_traceback)
 
-        if not self.test.result:
-            if not self.parent:
-                sys.exit(1)
+        if not self.parent:
+            sys.exit(0 if self.test.result else 1)
 
+        if not self.test.result:
             if isinstance(self.test.result, Fail):
                 result = Fail(test=self.parent.name, message=self.test.result.message)
             else:
@@ -945,6 +996,14 @@ class Suite(TestDefinition):
         kwargs["type"] = TestType.Suite
         return super(Suite, cls).__new__(cls, name, **kwargs)
 
+class Outline(TestDefinition):
+    """Outline definition."""
+    type = TestType.Outline
+
+    def __new__(cls, name=None, **kwargs):
+        kwargs["type"] = kwargs.pop("type", cls.type)
+        return super(Outline, cls).__new__(cls, name, **kwargs)
+
 class Test(TestDefinition):
     """Test definition."""
     type = TestType.Test
@@ -1049,7 +1108,6 @@ class NullStep():
 # decorators
 class TestDecorator(object):
     type = Test
-    outline = False
 
     def __init__(self, func):
         self.func = func
@@ -1061,9 +1119,9 @@ class TestDecorator(object):
 
         kwargs = dict(vars(self.func))
 
-        self.outline = kwargs.pop("outline", False)
         signature = inspect.signature(self.func)
-        kwargs["args"] = {p.name: p.default for p in signature.parameters.values() if p.default != inspect.Parameter.empty}
+        kwargs["args"] = {p.name: p.default for p in signature.parameters.values() if
+                          p.default != inspect.Parameter.empty}
 
         self.func.kwargs = kwargs
 
@@ -1085,24 +1143,66 @@ class TestDecorator(object):
             return r
 
         def run(test):
-            if self.outline and not args and test.examples:
-                for example in test.examples:
-                    if test.type == TestType.Iteration:
-                        type = test.parent_type
-                    else:
-                        type = test.type
-                    with Example(name=example, args=vars(example), type=type) as _example:
-                        process_func_result(self.func(_example, **vars(example)))
-                r = test
-            else:
-                r = process_func_result(self.func(test, **args))
-            return r
+            return process_func_result(self.func(test, **args))
 
-        if test is None or (test is not None and test.type > self.type.type):
+        if (test is None
+                or (test and test.type > self.type.type and self.type.type != TestType.Outline)
+                or (test and getattr(test, "_run_outline", False))):
             kwargs = dict(self.func.kwargs)
-            kwargs["args"] = dict(kwargs.get("args", {}))
-            kwargs.pop("test", None)
-            return self.type(**kwargs, test=self)(**args)
+
+            if isinstance(self, TestOutline):
+                no_arguments = (not args or getattr(test, "_run_outline_with_no_arguments", False))
+
+                if no_arguments and getattr(test, "examples", self.examples):
+                    kwargs["args"] = {}
+                    kwargs.pop("test", None)
+
+                    _test_type = self.type(**kwargs, test=self)
+
+                    def execute_examples():
+                        for example in getattr(test, "examples", self.examples):
+                            _kwargs = dict(self.func.kwargs)
+                            _kwargs["name"] = str(example)
+                            _kwargs["args"] = dict(kwargs.get("args", {}))
+                            _kwargs["args"].update(vars(example))
+                            _kwargs.update(dict(example._args))
+                            _kwargs.pop("type", None)
+                            _kwargs.pop("examples", None)
+
+                            if _test.type == TestType.Iteration:
+                                type = _test.parent_type
+                            else:
+                                type = _test.type
+
+                            _example_type = Example(type=type, **_kwargs)
+
+                            def execute_example(**args):
+                                process_func_result(self.func(current(), **args))
+
+                            _example_type.repeatable_func = execute_example
+
+                            with _example_type as _example:
+                                execute_example(**vars(example))
+
+                    _test_type.repeatable_func = execute_examples
+
+                    if test and getattr(test, "_run_outline", False):
+                        _test = test
+                        execute_examples()
+                    else:
+                        with _test_type as _test:
+                            execute_examples()
+
+                    return _test
+                else:
+                    if test and getattr(test, "_run_outline", False):
+                        return run(test)
+                    else:
+                        kwargs.pop("test", None)
+                        return self.type(**kwargs, test=self)(**args)
+            else:
+                kwargs.pop("test", None)
+                return self.type(**kwargs, test=self)(**args)
         else:
             return run(test)
 
@@ -1134,6 +1234,38 @@ class TestStep(TestDecorator):
 
         return self.__run__(**kwargs)
 
+class TestOutline(TestDecorator):
+    type = Outline
+
+    def __init__(self, func_or_type=None):
+        self.func = None
+        self.examples = None
+
+        if inspect.isfunction(func_or_type):
+            self.func = func_or_type
+        elif func_or_type is not None:
+            self.type = func_or_type
+
+        if self.func:
+            self._init_func()
+            TestDecorator.__init__(self, self.func)
+
+    def _init_func(self):
+        if getattr(self.func, "examples", None):
+            self.examples = self.func.examples
+
+    def __call__(self, *args, **kwargs):
+        if not self.func:
+            self.func = args[0]
+            self._init_func()
+            TestDecorator.__init__(self, self.func)
+            return self
+
+        if args:
+            raise TypeError(f"{self.func.__name__}() takes only named arguments")
+
+        return self.__run__(**kwargs)
+
 class TestCase(TestDecorator):
     type = Test
 
@@ -1158,83 +1290,7 @@ class TestBackground(TestDecorator):
             raise TypeError(f"{func.test} must be subclass of BackgroundTest")
         return super(TestBackground, self).__init__(func)
 
-class TestClass(object):
-    def __init__(self, test):
-        self.test = test
-
-    def __call__(self, func):
-        func.test = self.test
-        return func
-
-class ArgumentParser(object):
-    def __init__(self, parser):
-        self.argparser = parser
-
-    def __call__(self, func):
-        func.argparser = self.argparser
-        return func
-
-class Name(object):
-    def __init__(self, name):
-        self.name = name
-
-    def __call__(self, func):
-        func.name = self.name
-        return func
-
-class Description(object):
-    def __init__(self, description):
-        self.description = description
-
-    def __call__(self, func):
-        func.description = self.description
-        return func
-
-def Outline(func):
-    func.outline = True
-    return func
-
-class Examples(object):
-    def __init__(self, header, rows, row_format=None):
-        self.examples = ExamplesTable(header, rows=rows, row_format=row_format)
-
-    def __call__(self, func):
-        func.examples = self.examples
-        return func
-
-class Attributes(object):
-    def __init__(self, *attributes):
-        self.attributes = attributes
-
-    def __call__(self, func):
-        func.attributes = self.attributes
-        return func
-
-class Requirements(object):
-    def __init__(self, *requirements):
-        self.requirements = requirements
-
-    def __call__(self, func):
-        func.requirements = self.requirements
-        return func
-
-class Tags(object):
-    def __init__(self, *tags):
-        self.tags = tags
-
-    def __call__(self, func):
-        func.tags = self.tags
-        return func
-
-class Uid(object):
-    def __init__(self, uid):
-        self.uid = uid
-
-    def __call__(self, func):
-        func.uid = self.uid
-        return func
-
-def orders(tests):
+def ordered(tests):
     """Return ordered list of tests.
     """
     if settings.random_order:
@@ -1267,4 +1323,4 @@ def loads(name, *types, package=None, frame=None):
                 return True
             return member.type in types
 
-    return orders([test for name, test in inspect.getmembers(module, is_type)])
+    return ordered([test for name, test in inspect.getmembers(module, is_type)])
