@@ -13,20 +13,25 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 from testflows._core.test import Module, Suite, Test, Step, NullStep
-from testflows._core.test import TestStep, TestCase, TestSuite, TestModule, TestBackground
-from testflows._core.test import Attributes, Requirements, Examples, ArgumentParser
-from testflows._core.test import Name, Description, Uid, Tags, TestClass, Context, Outline
-from testflows._core.test import Feature, Background, Scenario, Example
+from testflows._core.test import TestStep, TestCase, TestSuite, TestModule, TestBackground, TestOutline
+from testflows._core.test import Context
+from testflows._core.test import Feature, Background, Scenario, Example, Outline
 from testflows._core.test import Given, When, Then, And, But, By, Finally
-from testflows._core.test import TestFeature, TestScenario, loads, orders
-from testflows._core.filters import the, thetags
+from testflows._core.test import TestFeature, TestScenario, loads, ordered
+from testflows._core.flags import Flags
+from testflows._core.objects import OK, XOK, Fail, XFail, Skip, Error, XError, Null, XNull
+from testflows._core.objects import Name, Description, Uid, Tags, Args
+from testflows._core.objects import XFails, XFlags, Repeat, RepeatTest
+from testflows._core.objects import Attributes, Requirements, Examples, ArgumentParser
+from testflows._core.objects import Node, Tag, Argument, Attribute, Requirement, Metric, Value, Ticket
+from testflows._core.baseobject import Table
+from testflows._core.filters import The, TheTags
 from testflows._core.funcs import top, current, previous, load, append_path
 from testflows._core.funcs import main, args, maps
 from testflows._core.funcs import metric, ticket, value, note, debug, trace
 from testflows._core.funcs import message, exception, ok, fail, skip, err
 from testflows._core.funcs import null, xok, xfail, xerr, xnull, pause, getsattr
-from testflows._core.funcs import xfails, xflags, tags, examples, table, repeat
-from testflows._core.funcs import current_dir, current_module, load_module 
+from testflows._core.funcs import current_dir, current_module, load_module
 from testflows._core.flags import TE, UT, SKIP, EOK, EFAIL, EERROR, ESKIP
 from testflows._core.flags import XOK, XFAIL, XERROR, XNULL
 from testflows._core.flags import FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NOT_COUNTED
